@@ -1,18 +1,23 @@
 #!/usr/bin/env python
+import logging
+
+import vobject.icalendar
+
+__version__ = "1.0.0"
 
 from .davclient import DAVClient
-## TODO: I don't think I like the practice of "import *" here.  Should be revisited prior to launching 1.0 at least.
 from .objects import *
 
-# possibly a bug in the tBaxter fork of vobject, this one has to be
-# imported explicitly to make sure the attribute behaviour gets
-# correctly loaded:
-from vobject import icalendar
-import logging
+
+## Notes:
+##
+## * The vobject.icalendar has (or had?) to be explicitly imported due to some bug in the tBaxter fork of vobject.
+## * The "import *" looks quite ugly, should be revisited prior to launching 1.0.
 
 # Silence notification of no default logging handler
 log = logging.getLogger("caldav")
 
+
 class NullHandler(logging.Handler):
     def emit(self, record):
         pass
