@@ -13,7 +13,7 @@
 ## * Consider how to get this into the documentation
 incompatibility_description = {
     'no_expand':
-        """Server may throw errors when asked to do a expanded date search""",
+        """Server may throw errors when asked to do a expanded date search (this is ignored by the tests now, as we're doing client-side expansion)""",
 
     'no_recurring':
         """Server is having issues with recurring events and/or todos. """
@@ -21,13 +21,13 @@ incompatibility_description = {
         """and events/todos may not be expanded with recurrances""",
 
     'no_recurring_expandation':
-        """Server will not expand recurring events""",
+        """Server will not expand recurring events (this is ignored by the tests now, as we're doing client-side expansion)""",
 
     'no_recurring_todo':
         """Recurring events are supported, but not recurring todos""",
 
     'no_recurring_todo_expand':
-        """Recurring todos aren't expanded""",
+        """Recurring todos aren't expanded (this is ignored by the tests now, as we're doing client-side expansion)""",
 
     'no_scheduling':
         """RFC6833 is not supported""",
@@ -74,9 +74,6 @@ incompatibility_description = {
     'event_by_url_is_broken':
         """A GET towards a valid calendar object resource URL will yield 404 (wtf?)""",
 
-    'uid_required':
-        """Server will not accept calendar object resources without an UID""",
-
     'no_sync_token':
         """RFC6578 is not supported, things will break if we try to do a sync-token report""",
 
@@ -145,21 +142,60 @@ incompatibility_description = {
     'dav_not_supported':
         """when asked, the server may claim it doesn't support the DAV protocol.  Observed by one baikal server, should be investigated more (TODO) and robur""",
 
+    'category_search_yields_nothing':
+        """When querying for a text match report over fields like the category field, server returns nothing""",
+
+    'text_search_is_case_insensitive':
+        """Probably not supporting the collation used by the caldav library""",
+
+    'text_search_is_exact_match_only':
+        """Searching for 'CONF' i.e. in the class field will not yield CONFIDENTIAL.  Which generally makes sense, but the RFC specifies substring match""",
+
+    'text_search_is_exact_match_sometimes':
+        """Some servers are doing an exact match on summary field but substring match on category""",
+
+   'combined_search_not_working':
+        """When querying for a text match and a date range in the same report, weird things happen""",
+
+   'text_search_not_working':
+        """Text search is generally broken""",
+
+   'radicale_breaks_on_category_search':
+        """See https://github.com/Kozea/Radicale/issues/1125""",
+
+   'fastmail_buggy_noexpand_date_search':
+        """The 'blissful anniversary' recurrent example event is returned when asked for a no-expand date search for some timestamps covering a completely different date""",
+
     'non_existing_raises_other':
         """Robur raises AuthorizationError when trying to access a non-existing resource (while 404 is expected).  Probably so one shouldn't probe a public name space?""",
 
     'no_supported_components_support':
         """The supported components prop query does not work""",
+
+    'rrule_takes_no_count':
+        """Fastmail consistently yields a "502 bad gateway" when presented with a rrule containing COUNT""",
+
+    'no-current-user-principal':
+        """when querying for the current user principal property, server doesn't report anything useful""",
+
+    'read_only':
+        """The calendar server does not support PUT, POST, DELETE, PROPSET, MKCALENDAR, etc""",
+
+    'no_relships':
+        """The calendar server does not support child/parent relationships between calendar components""",
 }
 
 xandikos = [
-    ## TEMP TEMP TEMP - TODO - should be investigated
-    ## (perhaps my xandikos version is too old?)
+    ## https://github.com/jelmer/xandikos/issues/8
     "no_expand", "no_recurring",
 
+    'text_search_is_exact_match_only',
+
+    ## This one is fixed in master branch
+    'category_search_yields_nothing', ## https://github.com/jelmer/xandikos/pull/194
+
     ## scheduling is not supported
     "no_scheduling",
-
 ]
 
 radicale = [
@@ -171,12 +207,19 @@ radicale = [
     ## freebusy is not supported yet, but on the long-term road map
     "no_freebusy_rfc4791",
 
+    ## TODO: raise an issue on this one
+    "radicale_breaks_on_category_search",
+
     ## Expanding recurrent events is not yet supported
     ## ref https://github.com/Kozea/Radicale/issues/662
     "no_recurring_expandation",
 
     'no_scheduling',
 
+    'text_search_is_case_insensitive',
+    'text_search_is_exact_match_only',
+    'combined_search_not_working',
+
     ## extra features not specified in RFC5545
     "calendar_order",
     "calendar_color"
@@ -200,11 +243,12 @@ zimbra = [
     'no_displayname',
     'duplicate_in_other_calendar_with_same_uid_is_lost',
     'event_by_url_is_broken',
-    'uid_required',
     'no_todo_on_standard_calendar',
     'no_sync_token',
-    #'no_recurring_todo',
     'vtodo_datesearch_notime_task_is_skipped',
+    'category_search_yields_nothing',
+    'text_search_is_exact_match_only',
+    'no_relships',
 
     ## extra features not specified in RFC5545
     "calendar_order",
@@ -235,6 +279,8 @@ baikal = [
     'no_recurring_todo',
     'no_recurring_todo_expand',
     'non_existing_calendar_found',
+    'combined_search_not_working',
+    'text_search_is_exact_match_sometimes',
 
     ## extra features not specified in RFC5545
     "calendar_order",
@@ -279,19 +325,25 @@ nextcloud = [
     'sync_breaks_on_delete',
     'no_recurring_todo',
     'no_recurring_todo_expand',
+    'combined_search_not_working',
+    'text_search_is_exact_match_sometimes',
 ]
 
 fastmail = [
     'duplicates_not_allowed',
     'duplicate_in_other_calendar_with_same_uid_breaks',
     'no_todo',
-    'sticky_events'
+    'sticky_events',
+    'fastmail_buggy_noexpand_date_search',
+    'combined_search_not_working',
+    'text_search_is_exact_match_sometimes',
+    'rrule_takes_no_count'
 ]
 
 synology = [
     "fragile_sync_tokens",
     "vtodo_datesearch_notime_task_is_skipped",
-    "no_recurring_todo"
+    "no_recurring_todo",
 ]
 
 robur = [
@@ -301,9 +353,9 @@ robur = [
     'no_supported_components_support',
     'no_journal',
     'no_freebusy_rfc4791',
-    'dav_not_supported',
     'no_todo_datesearch', ## returns nothing
-    'no_recurring', ## See https://github.com/roburio/caldav/issues/28
+    'text_search_not_working',
+    'no_relships'
 ]
 
 
