@@ -48,10 +48,11 @@ caldav_servers = [
     { # 5
         ## Does not get a proper response on get-current-principal propfind.
         ## should do more research into it, maybe
-        'url': 'http://dav.cladmi.eu/cal.php/calendars/tbrox/default/',
+        'url': 'http://dav.cladmi.eu/cal.php',
+        'principal_url': '/cal.php/principals/tbrox/',
         'username': 'tbrox',
         'password': 'testdigestauthenticationtbrox',
-        #'incompatibilities': [ 'no_mkcalendar' ],
+        'incompatibilities': [ 'no-current-user-principal' ], ## which is very strange, because from the web browser it does show this property
         'enable': False ## mkcalendar fails, calendar.events() fails ... but login works.  Should probably check more on this.  Hm.  It's some sabre thing, and it's possible to create new calendar through the web ui.  weird.
     },
     { # 6
@@ -88,18 +89,30 @@ caldav_servers = [
         'username': 'caldavtest',
         'password': 'tycteucbok',
         'incompatibilities': compatibility_issues.robur
+    },
+    { #11 - reimer, telegram, issue 201 / 74196
+        'url': 'https://dav.solox.de/p/Daniel.Reimer',
+        'username': 'Daniel.Reimer',
+        'password': 'solox',
+        'enable': False,
+        'incompatibilities': ['no_scheduling', "propfind_allprop_failure", "read_only"],
     }
 ]
 
 if False:
-    for x in caldav_servers:
-        x['enable'] = False
-    caldav_servers[10]['enable']=True
-
     test_xandikos = True
     test_radicale = True
+elif False:
+    caldav_servers = [{'url': 'http://localhost:8080', 'username': 'user', 'password': 'nopass'}]
+elif False:
+    for x in caldav_servers:
+        x['enable'] = False
+    caldav_servers[2]['enable']=True
+
+    test_xandikos = False
+    test_radicale = False
 else:
-    #caldav_servers = []
+    caldav_servers = []
     test_xandikos = True
     test_radicale = True
 
