@@ -5,9 +5,11 @@ Rule: None of the tests in this file should initiate any internet
 communication, and there should be no dependencies on a working caldav
 server for the tests in this file.  We use the Mock class when needed
 to emulate server communication.
-
 """
+import pickle
+from datetime import date
 from datetime import datetime
+from datetime import timedelta
 
 import caldav
 import icalendar
@@ -21,9 +23,8 @@ from caldav.elements import dav
 from caldav.elements import ical
 from caldav.lib import error
 from caldav.lib import url
-from caldav.lib import vcal
-from caldav.lib.python_utilities import to_local
-from caldav.lib.python_utilities import to_str
+from caldav.lib.python_utilities import to_normal_str
+from caldav.lib.python_utilities import to_wire
 from caldav.lib.url import URL
 from caldav.objects import Calendar
 from caldav.objects import CalendarObjectResource
@@ -44,7 +45,7 @@ else:
     from urlparse import urlparse
     import mock
 
-## Some example icalendar data copied from test_caldav.py
+## Some example icalendar data partly copied from test_caldav.py
 ev1 = """BEGIN:VCALENDAR
 VERSION:2.0
 PRODID:-//Example Corp.//CalDAV Client//EN
@@ -72,6 +73,36 @@ STATUS:NEEDS-ACTION
 END:VTODO
 END:VCALENDAR"""
 
+todo_implicit_duration = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Example Corp.//CalDAV Client//EN
+BEGIN:VTODO
+UID:20070313T123432Z-456553@example.com
+DTSTAMP:20070313T123432Z
+DTSTART;VALUE=DATE:20070425
+DUE;VALUE=DATE:20070501
+SUMMARY:Submit Quebec Income Tax Return for 2006
+CLASS:CONFIDENTIAL
+CATEGORIES:FAMILY,FINANCE
+STATUS:NEEDS-ACTION
+END:VTODO
+END:VCALENDAR"""
+
+todo_explicit_duration = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Example Corp.//CalDAV Client//EN
+BEGIN:VTODO
+UID:20070313T123432Z-456553@example.com
+DTSTAMP:20070313T123432Z
+DTSTART:20070425T160000Z
+DURATION:P5D
+SUMMARY:Submit Quebec Income Tax Return for 2006
+CLASS:CONFIDENTIAL
+CATEGORIES:FAMILY,FINANCE
+STATUS:NEEDS-ACTION
+END:VTODO
+END:VCALENDAR"""
+
 journal = """
 BEGIN:VCALENDAR
 VERSION:2.0
@@ -89,6 +120,39 @@ END:VJOURNAL
 END:VCALENDAR
 """
 
+# example from http://www.rfc-editor.org/rfc/rfc5545.txt
+evr = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Example Corp.//CalDAV Client//EN
+BEGIN:VEVENT
+UID:19970901T130000Z-123403@example.com
+DTSTAMP:19970901T130000Z
+DTSTART;VALUE=DATE:19971102
+SUMMARY:Our Blissful Anniversary
+TRANSP:TRANSPARENT
+CLASS:CONFIDENTIAL
+CATEGORIES:ANNIVERSARY,PERSONAL,SPECIAL OCCASION
+RRULE:FREQ=YEARLY
+END:VEVENT
+END:VCALENDAR"""
+
+todo6 = """
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Example Corp.//CalDAV Client//EN
+BEGIN:VTODO
+UID:19920901T130000Z-123408@host.com
+DTSTAMP:19920901T130000Z
+DTSTART:19920415T133000Z
+DUE:19920516T045959Z
+SUMMARY:Yearly Income Tax Preparation
+RRULE:FREQ=YEARLY
+CLASS:CONFIDENTIAL
+CATEGORIES:FAMILY,FINANCE
+PRIORITY:1
+END:VTODO
+END:VCALENDAR"""
+
 
 def MockedDAVResponse(text):
     """
@@ -112,6 +176,73 @@ def MockedDAVClient(xml_returned):
     return client
 
 
+class TestExpandRRule:
+    """
+    Tests the expand_rrule method
+    """
+
+    def setup_method(self):
+        cal_url = "http://me:hunter2@calendar.example:80/"
+        client = DAVClient(url=cal_url)
+        self.yearly = Event(client, data=evr)
+        self.todo = Todo(client, data=todo6)
+
+    def testZero(self):
+        ## evr has rrule yearly and dtstart DTSTART 1997-11-02
+        ## This should cause 0 recurrences:
+        self.yearly.expand_rrule(start=datetime(1998, 4, 4), end=datetime(1998, 10, 10))
+        assert len(self.yearly.icalendar_instance.subcomponents) == 0
+
+    def testOne(self):
+        self.yearly.expand_rrule(
+            start=datetime(1998, 10, 10), end=datetime(1998, 12, 12)
+        )
+        assert len(self.yearly.icalendar_instance.subcomponents) == 1
+        assert not "RRULE" in self.yearly.icalendar_component
+        assert "UID" in self.yearly.icalendar_component
+        assert "RECURRENCE-ID" in self.yearly.icalendar_component
+
+    def testThree(self):
+        self.yearly.expand_rrule(
+            start=datetime(1996, 10, 10), end=datetime(1999, 12, 12)
+        )
+        assert len(self.yearly.icalendar_instance.subcomponents) == 3
+        data1 = self.yearly.icalendar_instance.subcomponents[0].to_ical()
+        data2 = self.yearly.icalendar_instance.subcomponents[1].to_ical()
+        assert data1.replace(b"199711", b"199811") == data2
+
+    def testThreeTodo(self):
+        self.todo.expand_rrule(start=datetime(1996, 10, 10), end=datetime(1999, 12, 12))
+        assert len(self.todo.icalendar_instance.subcomponents) == 3
+        data1 = self.todo.icalendar_instance.subcomponents[0].to_ical()
+        data2 = self.todo.icalendar_instance.subcomponents[1].to_ical()
+        assert data1.replace(b"19970", b"19980") == data2
+
+    def testSplit(self):
+        self.yearly.expand_rrule(
+            start=datetime(1996, 10, 10), end=datetime(1999, 12, 12)
+        )
+        events = self.yearly.split_expanded()
+        assert len(events) == 3
+        assert len(events[0].icalendar_instance.subcomponents) == 1
+        assert (
+            events[1].icalendar_component["UID"]
+            == "19970901T130000Z-123403@example.com"
+        )
+
+    def test241(self):
+        """
+        Ref https://github.com/python-caldav/caldav/issues/241
+
+        This seems like sort of a duplicate of testThreeTodo, but the ftests actually started failing
+        """
+        assert len(self.todo.data) > 128
+        self.todo.expand_rrule(
+            start=datetime(1997, 4, 14, 0, 0), end=datetime(2015, 5, 14, 0, 0)
+        )
+        assert len(self.todo.data) > 128
+
+
 class TestCalDAV:
     """
     Test class for "pure" unit tests (small internal tests, testing that
@@ -778,6 +909,33 @@ END:VCALENDAR
         calendar = Calendar(parent=calhome)
         assert calendar.client == calhome.client
 
+    def testData(self):
+        """
+        Event.data should always return a unicode string, without \r
+        Event.wire_data should always return a byte string, with \r\n
+        """
+        cal_url = "http://me:hunter2@calendar.example:80/"
+        client = DAVClient(url=cal_url)
+        my_event = Event(client, data=ev1)
+        ## bytes on py3, normal string on py2 (but nobody uses python2, I hope?)
+        bytestr = b"".__class__
+        assert isinstance(my_event.data, str)
+        assert isinstance(my_event.wire_data, bytestr)
+        ## this may have side effects, as it converts the internal storage
+        my_event.icalendar_instance
+        assert isinstance(my_event.data, str)
+        assert isinstance(my_event.wire_data, bytestr)
+        ## this may have side effects, as it converts the internal storage
+        my_event.vobject_instance
+        assert isinstance(my_event.data, str)
+        assert isinstance(my_event.wire_data, bytestr)
+        my_event.wire_data = to_wire(ev1)
+        assert isinstance(my_event.data, str)
+        assert isinstance(my_event.wire_data, bytestr)
+        my_event.data = to_normal_str(ev1)
+        assert isinstance(my_event.data, str)
+        assert isinstance(my_event.wire_data, bytestr)
+
     def testInstance(self):
         cal_url = "http://me:hunter2@calendar.example:80/"
         client = DAVClient(url=cal_url)
@@ -788,14 +946,91 @@ END:VCALENDAR
         icalobj.subcomponents[0]["SUMMARY"] = "yet another summary"
         assert my_event.vobject_instance.vevent.summary.value == "yet another summary"
         ## Now the data has been converted from string to vobject to string to icalendar to string to vobject and ... will the string still match the original?
-        lines_now = my_event.data.split("\r\n")
-        lines_orig = ev1.replace("Bastille Day Party", "yet another summary").split(
-            "\n"
+        lines_now = my_event.data.strip().split("\n")
+        lines_orig = (
+            ev1.replace("Bastille Day Party", "yet another summary").strip().split("\n")
         )
         lines_now.sort()
         lines_orig.sort()
         assert lines_now == lines_orig
 
+    def testComponent(self):
+        cal_url = "http://me:hunter2@calendar.example:80/"
+        client = DAVClient(url=cal_url)
+        my_event = Event(client, data=ev1)
+        icalcomp = my_event.icalendar_component
+        icalcomp["SUMMARY"] = "yet another summary"
+        assert my_event.vobject_instance.vevent.summary.value == "yet another summary"
+        ## will the string still match the original?
+        lines_now = my_event.data.strip().split("\n")
+        lines_orig = (
+            ev1.replace("Bastille Day Party", "yet another summary").strip().split("\n")
+        )
+        lines_now.sort()
+        lines_orig.sort()
+        assert lines_now == lines_orig
+        ## Can we replace the component?  (One shouldn't do things like this in normal circumstances though ... both because the uid changes and because the component type changes - we're putting a vtodo into an Event class ...)
+        icalendar_component = icalendar.Todo.from_ical(todo).subcomponents[0]
+        my_event.icalendar_component = icalendar_component
+        assert (
+            my_event.vobject_instance.vtodo.summary.value
+            == "Submit Quebec Income Tax Return for 2006"
+        )
+
+    def testTodoDuration(self):
+        cal_url = "http://me:hunter2@calendar.example:80/"
+        client = DAVClient(url=cal_url)
+        my_todo1 = Todo(client, data=todo)
+        my_todo2 = Todo(client, data=todo_implicit_duration)
+        my_todo3 = Todo(client, data=todo_explicit_duration)
+        assert my_todo1.get_duration() == timedelta(0)
+        assert my_todo1.get_due() == date(2007, 5, 1)
+        assert my_todo2.get_duration() == timedelta(days=6)
+        assert my_todo2.get_due() == date(2007, 5, 1)
+        assert my_todo3.get_duration() == timedelta(days=5)
+        foo6 = my_todo3.get_due().strftime("%s") == "1177945200"
+        some_date = date(2011, 1, 1)
+
+        my_todo1.set_due(some_date)
+        assert my_todo1.get_due() == some_date
+
+        ## set_due has "only" one if, so two code paths, one where dtstart is actually moved and one where it isn't
+        my_todo2.set_due(some_date, move_dtstart=True)
+        assert my_todo2.icalendar_instance.subcomponents[0][
+            "DTSTART"
+        ].dt == some_date - timedelta(days=6)
+
+        ## set_duration at the other hand has 5 code paths ...
+        ## 1) DUE and DTSTART set, DTSTART as the movable component
+        my_todo1.set_duration(timedelta(1))
+        assert my_todo1.get_due() == some_date
+        assert my_todo1.icalendar_instance.subcomponents[0][
+            "DTSTART"
+        ].dt == some_date - timedelta(1)
+
+        ## 2) DUE and DTSTART set, DUE as the movable component
+        my_todo1.set_duration(timedelta(2), movable_attr="DUE")
+        assert my_todo1.get_due() == some_date + timedelta(days=1)
+        assert my_todo1.icalendar_instance.subcomponents[0][
+            "DTSTART"
+        ].dt == some_date - timedelta(1)
+
+        ## 3) DUE set, DTSTART not set
+        dtstart = my_todo1.icalendar_instance.subcomponents[0].pop("DTSTART").dt
+        my_todo1.set_duration(timedelta(2))
+        assert my_todo1.icalendar_instance.subcomponents[0]["DTSTART"].dt == dtstart
+
+        ## 4) DTSTART set, DUE not set
+        my_todo1.icalendar_instance.subcomponents[0].pop("DUE")
+        my_todo1.set_duration(timedelta(1))
+        assert my_todo1.get_due() == some_date
+
+        ## 5) Neither DUE nor DTSTART set
+        my_todo1.icalendar_instance.subcomponents[0].pop("DUE")
+        my_todo1.icalendar_instance.subcomponents[0].pop("DTSTART")
+        my_todo1.set_duration(timedelta(days=3))
+        assert my_todo1.get_duration() == timedelta(days=3)
+
     def testURL(self):
         """Exercising the URL class"""
         long_url = "http://foo:bar@www.example.com:8080/caldav.php/?foo=bar"
@@ -897,6 +1132,9 @@ END:VCALENDAR
             == URL("//www.example.com/bar/").canonical()
         )
 
+        # 10) pickle
+        assert pickle.loads(pickle.dumps(url1)) == url1
+
     def testFilters(self):
         filter = cdav.Filter().append(
             cdav.CompFilter("VCALENDAR").append(
@@ -918,54 +1156,6 @@ END:VCALENDAR
         if value is not None:
             raise Exception("This should have crashed")
 
-    def test_vcal_fixups(self):
-        """
-        There is an obscure function lib.vcal that attempts to fix up
-        known ical standard breaches from various calendar servers.
-        """
-        broken_ical = [
-            ## This first one contains duplicated DTSTART in the event data
-            """BEGIN:VCALENDAR
-X-EXPANDED:True
-X-MASTER-DTSTART:20200517T060000Z
-X-MASTER-RRULE:FREQ=YEARLY
-BEGIN:VEVENT
-DTSTAMP:20210205T101751Z
-UID:20200516T060000Z-123401@example.com
-DTSTAMP:20200516T060000Z
-SUMMARY:Do the needful
-DTSTART:20210517T060000Z
-DTEND:20210517T230000Z
-RECURRENCE-ID:20210517T060000Z
-END:VEVENT
-BEGIN:VEVENT
-DTSTAMP:20210205T101751Z
-UID:20200516T060000Z-123401@example.com
-DTSTAMP:20200516T060000Z
-SUMMARY:Do the needful
-DTSTART:20220517T060000Z
-DTEND:20220517T230000Z
-RECURRENCE-ID:20220517T060000Z
-END:VEVENT
-BEGIN:VEVENT
-DTSTAMP:20210205T101751Z
-UID:20200516T060000Z-123401@example.com
-DTSTAMP:20200516T060000Z
-SUMMARY:Do the needful
-DTSTART:20230517T060000Z
-DTEND:20230517T230000Z
-RECURRENCE-ID:20230517T060000Z
-END:VEVENT
-END:VCALENDAR"""
-        ]  ## todo: add more broken ical here
-
-        for ical in broken_ical:
-            ## This should raise error
-            with pytest.raises(vobject.base.ValidateError):
-                vobject.readOne(ical).serialize()
-            ## This should not raise error
-            vobject.readOne(vcal.fix(ical)).serialize()
-
     def test_calendar_comp_class_by_data(self):
         calendar = Calendar()
         for (ical, class_) in (
